package meerkat.pollingstation;

import com.google.common.util.concurrent.FutureCallback;
import meerkat.pollingstation.controller.callbacks.ScanCallback;
import meerkat.pollingstation.controller.callbacks.ScanDataCallback;
import meerkat.protobuf.PollingStation;
import meerkat.rest.ProtobufMessageBodyReader;
import meerkat.rest.ProtobufMessageBodyWriter;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.servlet.ServletContainer;

import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * Created by Laura on 3/20/2017.
 */
public class ReceiverScanHandler implements PollingStationScanner.Consumer{

    public final static String CALLBACKS_ATTRIBUTE_NAME = "controller";

    private final Server server;
    private final List<FutureCallback<PollingStation.ScannedData>> callbacks;

    private LinkedBlockingQueue<ScanCallback> queue;
    private volatile boolean shutDownHasBeenCalled;

    public ReceiverScanHandler(int port, String subAddress) {

        callbacks = new LinkedList<>();

        server = new Server(port);

        queue = new LinkedBlockingQueue<>();
        shutDownHasBeenCalled = false;

        ServletContextHandler servletContextHandler = new ServletContextHandler(server, subAddress);
        servletContextHandler.setAttribute(CALLBACKS_ATTRIBUTE_NAME, (LinkedBlockingQueue<ScanCallback>) queue);


        ResourceConfig resourceConfig = new ResourceConfig(ReceiverWebAPI.class);
        resourceConfig.register(ProtobufMessageBodyReader.class);
        resourceConfig.register(ProtobufMessageBodyWriter.class);

        ServletHolder servletHolder = new ServletHolder(new ServletContainer(resourceConfig));

        servletContextHandler.addServlet(servletHolder, "/*");


    }

    private boolean wasShutDownCalled () {
        return shutDownHasBeenCalled;
    }

    @Override
    public void start() throws Exception {
        server.start();



        while (! wasShutDownCalled()) {
            try {
                ScanDataCallback callback = (ScanDataCallback) queue.take();
//                handleSingleCommand(Command);
                callback.doNothing();
            }
            catch (InterruptedException e) {
                System.err.println("Interrupted while reading from command queue " + e);
            }
        }
    }

    @Override
    public void stop() throws Exception {
        server.stop();
    }

    @Override
    public void subscribe(FutureCallback<PollingStation.ScannedData> scanCallback) {
        callbacks.add(scanCallback);
    }

}